--Copyright (c) 2017 Stephen Ward (AukonDK) bluedrava.com

--This software is provided 'as-is', without any express or implied
--warranty. In no event will the authors be held liable for any damages
--arising from the use of this software.

--Permission is granted to anyone to use this software for any purpose,
--including commercial applications, and to alter it and redistribute it
--freely, subject to the following restrictions:

--1. The origin of this software must not be misrepresented; you must not
--   claim that you wrote the original software. If you use this software
--   in a product, an acknowledgment in the product documentation would be
--   appreciated but is not required.
--2. Altered source versions must be plainly marked as such, and must not be
--   misrepresented as being the original software.
--3. This notice may not be removed or altered from any source distribution.

local prism = {}
setmetatable(prism, {__index=_G})
_ENV = prism

font = require("font")
transx = 0
transy = 0
scale = 0
papercanvas = love.graphics.newCanvas(32*8,24*8)
inkcanvas = love.graphics.newCanvas(32*8,24*8)
spritecanvas = love.graphics.newCanvas(32*8,24*8)
papercanvas:setFilter("nearest","nearest")
inkcanvas:setFilter("nearest","nearest")
spritecanvas:setFilter("nearest","nearest")

shader = love.graphics.newShader[[
  uniform Image paper;
  uniform Image ink;


      vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords )
      {
      vec4 pixel = Texel(texture, texture_coords );
      
      vec4 inkpixel = Texel(ink, texture_coords );
      vec4 paperpixel = Texel(paper, texture_coords );
      
      if (pixel.r == 1.0){return inkpixel;}
      if (pixel.r == 0.0){return paperpixel;};
      }
  ]]

clear = function(self)
  love.graphics.setCanvas(self.spritecanvas)
  love.graphics.clear()
  love.graphics.setCanvas(self.inkcanvas)
  love.graphics.clear()
  love.graphics.setCanvas(self.papercanvas)
  love.graphics.clear()
  love.graphics.setCanvas()


end


draw = function(self)
  self:drawBorder()
  
  love.graphics.push()
  self:applyScale()
  love.graphics.setColor(0,0,0)
	love.graphics.rectangle("fill",0,0, 256,192)--background
  love.graphics.setColor(255,255,255)
  love.graphics.setShader(shader)
  shader:send("paper", self.papercanvas)
  shader:send("ink", self.inkcanvas)

  love.graphics.draw(self.spritecanvas)


  love.graphics.pop()  
  love.graphics.setShader()

  self:clear()
end

getColourNumber = function(input)
  --convert names of colours (red, cyan etc) to number for ease of use
  local conversion = {black = 0, blue = 1, red = 2, magenta = 3, green = 4, cyan = 5, yellow = 6, white = 7, trans = 9, rand = 8}
  local number = ""
  for col, num in pairs(conversion) do
    if input == col then
      number = num
    end
  end

  if number == "" then number = input end
  return number
end

getColour = function(colour, bright, alpha)

  local value = 0
  local rgb = {0,0,0}
  local number = getColourNumber(colour)
  bright = bright or false



  if number > 0 then

    if bright then
      value = 255
    else
      value = 215
    end

    if number == 1 then
      rgb = {0,0,value,alpha}
    elseif number == 2 then
      rgb = {value,0,0,alpha}
    elseif number == 3 then
      rgb = {value,0,value,alpha}
    elseif number == 4 then
      rgb = {0,value,0,alpha}
    elseif number == 5 then			
      rgb = {0,value,value,alpha}
    elseif number == 6 then
      rgb = {value,value,0,alpha}
    elseif number == 7 then
      rgb = {value,value,value,alpha}
    elseif number == 9 then
      rgb = {0,0,0,0}
    elseif number == 8 then
      rgb = {math.random(255),math.random(255),math.random(255),alpha}
    end

  end

  return rgb

end

calcScale = function(self)
  local winx,winy = love.window.getMode()
  local xres, yres = 256, 192
  local border = 1.25


  local xscale = winx/(xres*border)
  local yscale = winy/(yres*border)

  if (yscale*xres)*border > winx then
    self.scale=xscale
  else
    self.scale=yscale
  end
  self.transx, self.transy = winx/2 - (xres*self.scale)/2, winy/2 - (yres*self.scale)/2
end

applyScale=function(self)

  love.graphics.translate(self.transx, self.transy)
  love.graphics.scale(self.scale)


end

getMousePosition = function(self)
  local x,y = love.mouse.getPosition()

  x = (x-self.transx)/self.scale
  y = (y-self.transy)/self.scale

  return x, y
end

colourCells = function(x, y, w, h)

  local width = w
  local height = h
  local xstart = math.floor(x / 8)*8
  local xend = math.ceil((x + w) / 8)*8
  local ystart = math.floor(y / 8)*8
  local yend = math.ceil((y + h) / 8)*8
--  local xoffset = xstart - x

--  width = width + xoffset + (8-xoffset)
  width = xend - xstart
  height = yend - ystart


  love.graphics.rectangle("fill", xstart, math.floor(y / 8)*8, width, height)


end



sprite = function(sprite, x, y, ink, paper, bright, alpha,r)
  r = r or 0
  ink = ink or 7
  paper = paper or 0

  alpha = alpha or 255
  --start position
  local w, h = 0, 0

  --draw paper
  love.graphics.setCanvas(papercanvas)


  love.graphics.setColor(getColour(paper, bright, alpha)) 

--width is length of first row, height is number of rows
  colourCells(x,y, #sprite[1], #sprite)


--draw ink
  love.graphics.setCanvas(inkcanvas)


  love.graphics.setColor(getColour(ink, bright,alpha))
  colourCells(x,y, #sprite[1], #sprite)
--draw sprite
  love.graphics.setCanvas(spritecanvas)
  love.graphics.setColor(255,255,255,255)

  for _, row in ipairs(sprite) do
    for _, pixel in ipairs(row) do

      if pixel == 1 then
        --not using math floor makes rotation smoother... using it makes a more retro effect... not sure what I like best


        rx = math.floor(x+(w-#row/2)*math.cos(r) - (h-#sprite/2)*math.sin(r))+#row/2
        ry = math.floor(y+(w-#row/2)*math.sin(r) + (h-#sprite/2)*math.cos(r))+#sprite/2


        love.graphics.rectangle("fill", rx, ry, 1, 1)
      end

      w = w + 1

    end

    w = 0
    h = h + 1

  end

  love.graphics.setCanvas()

end

sprint = function(str, startx, starty, ink, paper, bright, alpha)

  local x, y = startx, starty

  ink = ink or 7
  paper = paper or 0
  bright = bright or false

  str = tostring(str) or "empty"

  for i in str:gmatch"." do
    if font[i] then
      sprite(font[i], x, y, ink, paper, bright, alpha)
    end
    --line feed (new line)
    if i == "\n" then
      x = startx
      y = y + 8
    else
      x = x + 8
    end
  end



end

borderColour = 2
borderMode = 1
borderTimer = 1
borderPos = 0
borderDuration = 0

changeBorder = function(mode, duration, colour)
  borderColour = colour or 2
  borderMode = mode or 1
  borderDuration = duration or 0
end

--need to combine timers and set sequences?
drawBorder = function(self)
  dt = love.timer.getDelta( )
  
  	if borderDuration > 0 then
		borderDuration = borderDuration - dt
	else
		borderDuration = 0
		changeBorder(1,0,2)
	end
  
  
  if borderTimer > 3 then
    borderTimer = 1
  else
    borderTimer = self.borderTimer + dt
  end
  
  if borderPos > 10 then
    borderPos = 0
  else
    borderPos = borderPos + dt*5
  end
  
  local winx,winy = love.window.getMode()
  if borderMode == 1 then
    --single colour
   
    love.graphics.setColor(getColour(borderColour, 1))
		love.graphics.rectangle("fill",0,0, winx,winy)
  elseif borderMode == 2 then
    --waiting: flashing red cyan
    local c = 2
		if borderTimer > 2 then
			c = 5
		end
		love.graphics.setColor(getColour(c, 1))
		love.graphics.rectangle("fill",0,0, winx,winy)
  elseif borderMode==3 then
    --header: scrolling lines
    local line = 1
		local c = 2
		while line < 25 do --maybe make table first then draw from that
			love.graphics.setColor(getColour(c, 1))
			love.graphics.rectangle("fill",0,line*50-150+borderPos*10, winx,50)
			line = line + 1
			if c == 2 then
				c=5
			else
				c=2
			end
		end
    
    
  elseif borderMode==4 then
    --data: yellow blue rapid
    love.graphics.setColor(getColour(6, 1))
		love.graphics.rectangle("fill",0,0, winx,winy)
		love.graphics.setColor(getColour(1, 1))
		local line = 1
		while line < 25 do
			love.graphics.rectangle("fill",0,math.random(winy), winx,math.random(30))
			line = line +1
		end

  
  end
  
end


return prism